#!/bin/bash

# 0 = install online, 1 = install offline      
# (folders 'offline_packages' and 'qDslrDashboard_V3.6.4_RPI4_Buster.tar.gz' must be in the same directory)
useLocal=0

# 0 = ignore libQt5QmlModels, 1 = extract libQt5QmlModels 
# (qmlmodelslib.tar.gz must be in the same directory)
installQtm=0

# 0 = don't configure WLAN, 1 = install access point
installAP=0
# WiFi hotspot settings (used only with -AP flag)
SSID="MyWIFI"         # Your SSID
PASSWORD="passwort"   # Your password

# 0 = no VNC, 1 = install VNC
installVNC=0

# Installation directory
INSTALL_DIR=~/qdslrdashboard

# Color for final output
TURQUOISE='\033[0;36m'
NC='\033[0m' # Reset color

for arg in "$@"; do
  if [[ "$arg" == "-offline" ]]; then
    useLocal=1
  elif [[ "$arg" == "-QTM" ]]; then
    installQtm=1
  elif [[ "$arg" == "-AP" ]]; then
    installAP=1
  elif [[ "$arg" == "-VNC" ]]; then
    installVNC=1
  fi
done

ARCH=$(dpkg --print-architecture)

# 0. Check architecture
if [[ "$ARCH" == "armhf" ]]; then
  echo "📷 32-bit version detected."
  echo "Installing qDslrDashboard V3.6.4 for Raspberry Pi OS 32Bit (armhf)"
  echo "If issues occur, please check the image. This script is built for Bullseye (Build 2025-05-06)"
  echo "Website: https://downloads.raspberrypi.com/raspios_oldstable_lite_armhf/images/raspios_oldstable_lite_armhf-2025-05-07/"
else
  echo "❌ Error: Not Raspberry Pi OS Bullseye 32Bit (armhf) – please download the correct version"
  echo "Website: https://downloads.raspberrypi.com/raspios_oldstable_lite_armhf/images/raspios_oldstable_lite_armhf-2025-05-07/"
  rm -rf "$INSTALL_DIR"
  exit 1
fi

# 1. Create installation directory and download qDslrDashboard if installing online
echo "📁 Creating installation directory..."

mkdir -p "$INSTALL_DIR"
if [ "$useLocal" -eq 1 ]; then
  if [ -f qDslrDashboard_V3.6.4_RPI4_Buster.tar.gz ]; then
    echo "✅️ Found qDslrDashboard_V3.6.4_RPI4_Buster.tar.gz in current directory"
  else
    echo "❌ Error: qDslrDashboard_V3.6.4_RPI4_Buster.tar.gz not found in current directory!"
    rm -rf "$INSTALL_DIR"
    exit 1
  fi
else
   echo "⬇️ Downloading qDslrDashboard V3.6.4 for RPI4 Buster..."
   wget -N https://files.lrtimelapse.com/dslrdashboard/V3.6.4/qDslrDashboard_V3.6.4_RPI4_Buster.tar.gz
fi

if [ "$useLocal" -eq 1 ]; then
  # 2. Check offline package directory
  echo "📦 Using local .deb packages from ./offline_packages/"
  if [ ! -d ./offline_packages ]; then
    echo "❌ Error: Directory ./offline_packages not found!"
    rm -rf "$INSTALL_DIR"
    exit 1
  fi
  # 3. Install required packages
  sudo dpkg -i ./offline_packages/*.deb || sudo apt-get install -f -y
else
  echo "🌐 Installing packages online via apt..."
  # 2. System update
  echo "🔄 Updating system..."
  sudo apt update && sudo apt upgrade -y
  
  # 3. Install required libraries
  echo "📦 Installing required libraries..."
  sudo apt install -y \
    libraw20 \
    libexiv2-27 \
    libusb-1.0-0 \
    libqt5serialport5 \
    libqt5svg5 \
    libqt5network5 \
    libqt5gui5 \
    libqt5widgets5 \
    libqt5core5a \
    libxcb-keysyms1 \
    libxcb-image0 \
    libxcb-shm0 \
    libxcb-icccm4 \
    libxcb-sync1 \
    libxcb-xfixes0 \
    libxcb-shape0 \
    libxcb-randr0 \
    libxcb-render-util0 \
    libxcb-xinput0 \
    unzip \
    wget \
    libopencv-features2d-dev \
    libopencv-core-dev \
    libopencv-imgproc-dev \
    libopencv-highgui-dev \
    libqt5bluetooth5 \
    qml-module-qtmultimedia \
    libqt5multimedia5 \
    libqt5multimediawidgets5 \
    libqt5multimedia5-plugins \
    hostapd \
    dnsmasq 
fi

# 4. Create symbolic link for libexiv2.so.26 pointing to libexiv2.so.27
echo "🔗 Creating symbolic link libexiv2.so.26 → libexiv2.so.27"
if [ ! -f /usr/lib/arm-linux-gnueabihf/libexiv2.so.26 ]; then
  sudo ln -s /usr/lib/arm-linux-gnueabihf/libexiv2.so.27 /usr/lib/arm-linux-gnueabihf/libexiv2.so.26
fi

# 5. Extract main application archive
echo "📦 Extracting qDslrDashboard_V3.6.4_RPI4_Buster.tar.gz..."
tar -xzf qDslrDashboard_V3.6.4_RPI4_Buster.tar.gz -C "$INSTALL_DIR"

cd "$INSTALL_DIR"

# 6. Optionally extract QT Models Library if desired and available
if [[ "$installQtm" -eq 1 ]]; then
  QML_ARCHIVE="qmlmodelslib.tar.gz"
  QT_DIR="$INSTALL_DIR/qDslrDashboard_V3.6.4_RPI4_Buster/QT_Libraries"
  if [[ -f "$QML_ARCHIVE" ]]; then
    echo "🧩 Extracting $QML_ARCHIVE into $QT_DIR..."
    mkdir -p "$QT_DIR"
    tar -xzf "$QML_ARCHIVE" -C "$QT_DIR"
  else
    echo "⚠️  Notice: $QML_ARCHIVE not found – skipping QT Models import."
  fi
fi

# 7. Make start script executable
echo "Creating desktop link"
chmod +x qDslrDashboard_V3.6.4_RPI4_Buster/qDslrDashboard.sh

# 8. Create desktop shortcut
echo "🖥️ Creating desktop shortcut..."
cat <<EOF > ~/Desktop/qDslrDashboard.desktop
[Desktop Entry]
Name=qDslrDashboard
Exec=/home/$USER/qdslrdashboard/qDslrDashboard_V3.6.4_RPI4_Buster/qDslrDashboard -platform xcb
Icon=camera
Type=Application
Categories=Graphics;
EOF
chmod +x ~/Desktop/qDslrDashboard.desktop

if [[ "$installVNC" -eq 1 ]]; then
  # 9. Enable and start VNC server via raspi-config
  echo "🔧 Enabling and starting VNC server..."
  sudo raspi-config nonint do_vnc 0
  sudo systemctl enable vncserver-x11-serviced.service
  sudo systemctl start vncserver-x11-serviced.service
fi

if [[ "$installAP" -eq 1 ]]; then
  # ---------------------------
  # Setup WiFi hotspot
  # ---------------------------
  
  echo "📡 Setting up WiFi hotspot..."
  
  # 1. Install required packages
  sudo apt install -y hostapd dnsmasq
  
  # 2. Stop services for reconfiguration
  sudo systemctl stop hostapd
  sudo systemctl stop dnsmasq
  
  # 3. Set static IP for wlan0
  echo "interface wlan0
  static ip_address=192.168.5.1/24
  nohook wpa_supplicant" | sudo tee /etc/dhcpcd.conf > /dev/null
  
  # 4. Configure dnsmasq
  sudo mv /etc/dnsmasq.conf /etc/dnsmasq.conf.orig
  
  cat <<EOF | sudo tee /etc/dnsmasq.conf
interface=wlan0
dhcp-range=192.168.5.10,192.168.5.50,255.255.255.0,24h
EOF
  
  # 5. Configure hostapd
  cat <<EOF | sudo tee /etc/hostapd/hostapd.conf
interface=wlan0
driver=nl80211
ssid=$SSID
hw_mode=g
channel=6
wmm_enabled=0
macaddr_acl=0
auth_algs=1
ignore_broadcast_ssid=0
wpa=2
wpa_passphrase=$PASSWORD
wpa_key_mgmt=WPA-PSK
rsn_pairwise=CCMP
EOF
  
  # 6. Link config to hostapd
  sudo sed -i 's|#DAEMON_CONF="|DAEMON_CONF="/etc/hostapd/hostapd.conf|' /etc/default/hostapd
  
  # 7. Enable IP forwarding
  sudo sed -i 's/#net.ipv4.ip_forward=1/net.ipv4.ip_forward=1/' /etc/sysctl.conf
  sudo sysctl -w net.ipv4.ip_forward=1
  
  # 8. Configure NAT
  sudo iptables -t nat -A POSTROUTING -o eth0 -j MASQUERADE
  sudo sh -c "iptables-save > /etc/iptables.ipv4.nat"
  
  # 9. Create systemd service for iptables
  cat <<EOF | sudo tee /etc/systemd/system/iptables-restore.service
[Unit]
Description=Restore iptables rules
Before=network.target

[Service]
Type=oneshot
ExecStart=/sbin/iptables-restore < /etc/iptables.ipv4.nat
RemainAfterExit=yes

[Install]
WantedBy=multi-user.target
EOF

  sudo systemctl enable iptables-restore.service
  
  # 10. Enable and start services
  sudo rfkill unblock wifi
  sudo systemctl unmask hostapd
  sudo systemctl enable hostapd
  sudo systemctl start hostapd
  sudo systemctl restart dnsmasq
  
  echo ""
  echo "✅ WiFi hotspot setup complete!"
  echo "📶 Network name (SSID): '$SSID'"
  echo "🔑 Password: '$PASSWORD'"
  echo "🌐 IP Address: 192.168.5.1"
  echo ""
  echo "👉 You can now connect your device to the '$SSID' WiFi network."
fi

echo ""
echo -e "✅ ${TURQUOISE}Installation and configuration complete!${NC}"

echo -e "${TURQUOISE}Recommended settings for Canon in qDslrDashboard:${NC}"
echo -e "${TURQUOISE} - Use Canon Local Storage: enabled${NC}"
echo -e "${TURQUOISE} - Use Nikon Highspeed Transfer: disabled${NC}"

read -p "Done"
